/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to demonstrate how to process a number of PDF documents in multiple threads environment.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

//Structure for thread handle data
typedef struct _FSDK_THREADDATA
{
	FSCRT_THREADHANDLER	threadHandler;	//Thread handler of Foxit SDK
	FSCRT_CALLBACK_THREADFINALIZE threadFinalize; //Save callbacki function address.
	string filename;
} FSDK_THREADDATA;

// Thread call function, get current thread id
FSCRT_THREAD Multiple_GetCurrentThread(FS_LPVOID clientData)
{
	#if defined(_MSC_VER)
		return (FSCRT_THREAD)GetCurrentThreadId();
	#elif defined(__GNUC__)
		return (FSCRT_THREAD)pthread_self();
	#endif
}

// Set finalize callback
void Multiple_SetThreadFinalizeCallback(FS_LPVOID clientData, FSCRT_CALLBACK_THREADFINALIZE callbackThreadFinalize)
{
	((FSDK_THREADDATA*)clientData)->threadFinalize = callbackThreadFinalize;
	return ;
}

//Thread procedure to process PDF document
void Multiple_Threads_Proc(void* param)
{
	//Define a string object to store log data
	string strLog("\r\n====== ");

	//Get PDF file name from parameter, and output to log
	string strFile = FSDK_GetFixFolder();

	FSDK_THREADDATA* threadData = (FSDK_THREADDATA*)param;
		
	strFile += "/";
	strFile += threadData->filename;
	strLog  +=threadData->filename;

	//Get current time
	unsigned int nStartTime = FSDK_GetClock();

	CAutoDocument autoPdfDoc(strFile.c_str(), "rb");
	//Load PDF file
	FS_RESULT ret = autoPdfDoc.LoadPDFDocument(NULL);
	if (FSCRT_ERRCODE_SUCCESS != ret)
	{
		return;
	}

	//Load successfully
	strLog += " success\r\n";
	//Get count of PDF pages
	FS_INT32 pageCount = 0;
	ret = FSPDF_Doc_CountPages(autoPdfDoc.GetDocument(), &pageCount);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		FS_INT32 pageIndex = 0;
		do
		{
			//Get a PDF page by index
			CAutoPDFPage autoPage;
			ret =autoPage.LoadPDFPage(autoPdfDoc.GetDocument(),pageIndex);

			//Move to next page
			pageIndex ++;
		} while (pageIndex < pageCount);
	}
	//Get current time
	unsigned int nEndTime = FSDK_GetClock();

	//Output elapse time into log
	char info[128];
	sprintf(info, "\tpage count: %d\r\n\telapsed time: %dms\r\n", pageCount, nEndTime - nStartTime);
	strLog += info;

	//Output total log string
	FSDK_OutputStringLog(strLog.c_str(), strLog.length());
	threadData->threadFinalize(Multiple_GetCurrentThread(NULL));
	if(threadData) delete threadData;
}

//Entrance of this example
int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	FSDK_THREADDATA* threadData = new FSDK_THREADDATA;
	if(!threadData) return 0;

	threadData->threadHandler.clientData = threadData;
	threadData->threadHandler.GetCurrentThread = Multiple_GetCurrentThread;
	threadData->threadHandler.SetThreadFinalizeCallback = Multiple_SetThreadFinalizeCallback;
	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("multiple_threads");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: multiple_threads\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	
	FSCRT_Library_SetThreadHandler(&threadData->threadHandler);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();
		return -1;
	}
	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	string outputFolder = FSDK_GetOutputFilesFolder("multiple_threads");

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		if(threadData) delete threadData;
		return -1;
	}

	string inputFilesPath = FSDK_GetFixFolder();
	FSDK_OutputLog("Input from folder: %s\r\n", inputFilesPath.c_str());
	FSDK_OutputLog("Output to folder: %s\r\n", logPath.c_str());
	//Get folder of inputting files for this example
	FSDK_OutputLog("\r\nEnumerate files in folder input_files/multiple_threads: ");
	//string inputFilesPath = FSDK_GetInputFilesFolder("multiple_threads");
	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(inputFilesPath.c_str());
	if (folder)
	{
		//Array to store thread handles
		vector<void*> threads;

		do
		{
			//Get a file object from folder
			FS_BOOL isDir = FALSE;
			string strFile = FSDK_GetNextFile(folder, &isDir);
			if (strFile.size() > 0 && !isDir)
			{
				//Check whether is a PDF file
				if (FSDK_CompareExtName(strFile.c_str(), ".pdf"))
				{
					//Allocate memory block to store file name
					FSDK_THREADDATA* subThreadData = new FSDK_THREADDATA;
											
					subThreadData->filename = strFile;
					subThreadData->threadFinalize = threadData->threadFinalize;

					//Create a new thread to parse PDF file, pass file name as parameter
					void* thread = FSDK_CreateThread(Multiple_Threads_Proc, subThreadData);
					if (thread)
					{
						//Add thread handle into array, and go on for next file
						threads.push_back(thread);
						continue;
					}
					else
					{
						delete subThreadData;
					}
				}

				//Go on for next file
				continue;
			}

			//If it's an invalid file name
			if (strFile.size() < 1) break;
		} while (1);
		//Close folder
		FSDK_CloseFolder(folder);

		if (threads.size() > 0)
		{
			//Wait for exit status of all threads
			FSDK_WaitForMultipleThreads(&threads.front(), threads.size());
		}
		else
		{
			//No file found, output log string
			FSDK_OutputLog("\r\nNo file found in folder\r\n");
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	if(threadData) delete threadData;
	return 0;
}
