/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is a demo to extract the text from PDF files and save them as text files.
 **/

#include "../comm_src/fgsdk_common.h"
#include "pdf2text.h"

#if defined(FSDK_IOS_DEMO)
#define main pdf2text
#endif

//Covert a PDF file to a text file
CPDFTransToText::CPDFTransToText()
	:m_strDestFolder(""),
	m_strPageRange(""),
	m_strSrcFile("")
{

}

CPDFTransToText::~CPDFTransToText()
{

}

void CPDFTransToText::PDFToText(string fileName, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("\r\n====== Convert %s to TXT ======\r\n", fileName.c_str());

	//Prepare for output file name and the format is xxxx.txt.
	//Abstract file name except the extension name
	if (fileName.length() <= 4)
	{
		FSDK_OutputLog("Failure: The file name is invalid. Please check it.\r\n");
		return;
	}
	
	//Open an input PDF file
	string strPdfFile = inputFilesPath + "/" + fileName;	
	CAutoDocument autoInputDoc(strPdfFile.c_str(), "rb");

	//Load a PDF document object from a file
	if (FSCRT_ERRCODE_SUCCESS != autoInputDoc.LoadPDFDocument(NULL))
	{
		return;
	}

	//Compose the output file
	string strTxtFile = outputFilesPath + "/";
	strTxtFile += string(fileName.c_str(), fileName.length()-4) + ".txt";
	
	//Open an output txt file
	CAutoDocument autoOutPutDoc(strTxtFile.c_str(), "wb+");

	//Mark the starting time
	unsigned int nStartTime = FSDK_GetClock();
	FS_RESULT ret = FSCRT_ERRCODE_ERROR;
	FS_INT32 nPageCount = 0;
	FS_INT32 nPageNum = 0;
	FS_INT32* nPageIndexArray = NULL;
	if (m_strPageRange != "")
	{
		if (FSDK_GetPageRange(m_strPageRange.c_str(), nPageIndexArray, &nPageNum))
		{
			//Get page count
			ret = FSPDF_Doc_CountPages(autoInputDoc.GetDocument(), &nPageCount);
			if (ret != FSCRT_ERRCODE_SUCCESS)
			{
				FSDK_OutputLog("Failure: Failed to get page count with error code %d.\r\n", ret);
				if (nPageIndexArray)
				{
					delete[] nPageIndexArray;
					nPageIndexArray = NULL;
				}
				return ;
			}
		}
	}
	if (nPageNum < 1)
		nPageNum = 1;

	//Parse and convert PDF page one by one into one text file 
	FS_INT32 nFinishPageCount = 0;
	for (int i=0; i < nPageNum; i++)
	{
		FS_INT32 pageIndex;
		if (m_strPageRange != "" && nPageIndexArray)
		{
			if (nPageIndexArray[i] >= nPageCount || nPageIndexArray[i] < 0)
			{
				FSDK_OutputLog("Failure: Page index %d is out of valid range!\r\n", nPageIndexArray[i]);
				continue;
			}
			pageIndex = nPageIndexArray[i];
		}
		else
		{
			pageIndex = i;
			FSDK_OutputLog("\r\n");
		}

		CAutoPDFPage autoPage;
		//Get a page based on the given page index
		ret = autoPage.LoadPDFPage(autoInputDoc.GetDocument(),pageIndex);
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}

		//Before loading a text page from a PDF page, the PDF page should be pared successfully first.
		if (FSCRT_ERRCODE_SUCCESS != autoPage.ParsePage())
		{
			continue;
		}
		//Load a text page based on a PDF page
		ret = autoPage.LoadTextPage();
		if (FSCRT_ERRCODE_SUCCESS != ret)
		{
			continue;
		}

		//Export a text page to the given text file
		ret = FSPDF_TextPage_ExportToFile(autoPage.GetPDFTextPage(),autoOutPutDoc.GetFileHander());
		if(ret != FSCRT_ERRCODE_SUCCESS)
		{
			FSDK_OutputLog("Failure: In function FSPDF_TextPage_ExportToFile with error code %d.\r\n", ret);
			continue;
		}
		FSDK_OutputLog("Success: Convert to TXT form page with index %d.\r\n", i);
		nFinishPageCount++;
	}
	if (nPageIndexArray)
	{
		delete []nPageIndexArray;
		nPageIndexArray = NULL;
	}

	//Mark the end time
	unsigned int nEndTime = FSDK_GetClock();
	//Record the duration
	FSDK_OutputLog("\tfinish page count: %d\r\n\telapsed time: %dms\r\n", nFinishPageCount, nEndTime - nStartTime);
}

void  CPDFTransToText::BrowseFolder(void* folder, string inputFilesPath, string outputFilesPath)
{
	FSDK_OutputLog("Enter the input folder\r\n");

	do
	{
		FS_BOOL bIsDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &bIsDir);
		if(!bIsDir)
		{
			// Check whether a file is PDF or not
			if(FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
				PDFToText(strFile, inputFilesPath, outputFilesPath);
		}
		if (strFile.size() < 1) break;

	} while (TRUE);
}

int main(int argc, char* argv[])
{
	FSDK_SetExecuteFilePath(argc, argv);

	CPDFTransToText Pdf2Text;
	//Judge if to command line arguments. 
	//usage: pdf2text <srcfile> /r [pagerange] /o [destfolder]
	if (argc > 1)
	{
		Pdf2Text.m_strSrcFile = argv[1];
		bool bArgumentValid = true;
		switch(argc)
		{
		case 2:
			break;
		case 3:
			{
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				break;
			}
		case 4:
			{//usage: pdf2text <srcfile> XXX XXX
				if (0 != strcmp(argv[2], "/r") && 0 != strcmp(argv[2], "/o"))
					bArgumentValid = false;
				else if (0 == strcmp(argv[2], "/r"))
				{
					if (0 != strcmp(argv[3], "/o") && '/'!=argv[3][0])
						Pdf2Text.m_strPageRange = argv[3]; //usage: pdf2text <srcfile> /r [pagerange]
					else if (0 != strcmp(argv[3], "/o") && '/'==argv[3][0])
						bArgumentValid = false;
				}
				else if (0 == strcmp(argv[2], "/o"))
					Pdf2Text.m_strDestFolder = argv[3]; //usage: pdf2text <srcfile> /o [destfolder]
				break;
			}
		case 5:
			{//usage: pdf2text <srcfile> /r XXX XXX
				if (0 != strcmp(argv[2], "/r"))
					bArgumentValid = false;
				else 
				{
					if (0 != strcmp(argv[3], "/o"))
					{
						if (0 != strcmp(argv[4], "/o"))
							bArgumentValid = false;
						else
							Pdf2Text.m_strPageRange = argv[3]; //usage: pdf2text <srcfile> /r [pagerange] /o
					}
					else
						Pdf2Text.m_strDestFolder = argv[4];//usage: pdf2text <srcfile> /r /o [destfolder]
				}
				break;
			}
		case 6:
			{//usage: pdf2text <srcfile> /r [pagerange] /o [destfolder]
				if (0 != strcmp(argv[2], "/r") || 0 != strcmp(argv[4], "/o"))
					bArgumentValid = false;
				else
				{
					Pdf2Text.m_strPageRange = argv[3];
					Pdf2Text.m_strDestFolder = argv[5];
				}
				break;
			}
		default:
			bArgumentValid = false;
		}
		if (false == bArgumentValid || !FSDK_FileExist(Pdf2Text.m_strSrcFile.c_str()) || (Pdf2Text.m_strDestFolder != "" && !FSDK_FolderExist(Pdf2Text.m_strDestFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdf2text [<srcfile> /r [pagerange] /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of a PDF file as input file.\r\n");
			printf("/r [pagerange]\t: Page range, used to specify indexes of PDF pages in input PDF\r\n ");
			printf("\t\t  file, such as \"0\", \"1-3,5\". Page index starts from 0 and\r\n\t\t  should be valid within page count. Invalid index will be\r\n\t\t  ignored. "\
				"If input page range is totally invalid, page range is\r\t\t  to be first page by default.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}

	// Locate the path of log file
	string logPath ;
	if (Pdf2Text.m_strDestFolder != "")
	{
		Pdf2Text.m_strDestFolder += "/pdf2text";
		FSDK_CreatFolder(Pdf2Text.m_strDestFolder.c_str());
		logPath = Pdf2Text.m_strDestFolder;
		logPath += "/log.txt";
	}
	else
	{
		Pdf2Text.m_strDestFolder = FSDK_GetOutputFilesFolder("pdf2text");
		logPath = FSDK_GetLogFile("pdf2text");
	}

	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdf2text\r\n\r\n");

	//Initialize SDK library, or no method of SDK can be used.
	FS_BOOL bRet = FSDK_InitializeLibray(TRUE);
	if (!bRet)
	{
		FSDK_CloseLog();
		return 0;
	}
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}
	
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(Pdf2Text.m_strDestFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}


	string inputFolder;
	string pdfName;

	if (Pdf2Text.m_strSrcFile != "")
	{
		pdfName = FSDK_GetFileName(Pdf2Text.m_strSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(Pdf2Text.m_strSrcFile.c_str());
		FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder: %s\r\n", Pdf2Text.m_strDestFolder.c_str());
		Pdf2Text.PDFToText(pdfName, inputFolder, Pdf2Text.m_strDestFolder);	
	}
	else
	{
		// Locate the folder where input files are
		inputFolder = FSDK_GetInputFilesFolder("pdf2text");		
		// Open the folder
		void* folder = FSDK_OpenFolder(inputFolder.c_str(), false);
		if (folder)
		{
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", Pdf2Text.m_strDestFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdf2text: ");
			//Browse the given folder and deal with files one by one
			Pdf2Text.BrowseFolder(folder, inputFolder, Pdf2Text.m_strDestFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			pdfName = "AboutFoxit.pdf";
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder: %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder: %s\r\n", Pdf2Text.m_strDestFolder.c_str());
			Pdf2Text.PDFToText(pdfName, inputFolder, Pdf2Text.m_strDestFolder);	
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();

	return 0;
}
