/**
* Copyright (C) 2001-2017, Foxit Software Inc..
* All Rights Reserved.
*
* http://www.foxitsoftware.com
*
* The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
* You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
* unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
*
* This demo illustrates how to add and verify signatures in PDF files by Foxit PDF SDK.
*/

//Include common header file.
#include "../comm_src/fgsdk_common.h"
#include "signature.h"

#ifdef WIN32 || _WIN64
extern "C"
{
#include <openssl/applink.c>
};
#pragma  comment(lib,"ws2_32.lib")
#pragma  comment(lib,"../../../lib/libeay32.lib")
#endif

#include <sstream>

#if defined(FSDK_IOS_DEMO)
#define main pdfdigitalsignature
#endif

/*******************************************************************************/
/*  Helper class.															   */
/*******************************************************************************/

// FSCRT_DigestData
class FSCRT_DigestData
{
public:
	FSCRT_DigestData(){}
	~FSCRT_DigestData(){}
public:
	FSPDF_SIGNATURE m_pSig;
	FSCRT_FILE m_file;
	const FS_DWORD* m_pByteRangeArray;
	FS_DWORD m_sizeofArray;
};

//CFSCRT_DigestContext
class CFSCRT_DigestContext
{
public:
	CFSCRT_DigestContext();
	~CFSCRT_DigestContext();
public:
	//Initialize and Finalize functions.
	void	Initialize();
	void	Release();

	//Set variable m_pDigestData.
	FS_BOOL	SetData(FSCRT_FILE file, const FS_DWORD* byteRangeArray, FS_DWORD sizeofArray);
	//Get variable m_pDigestData.
	FS_BOOL	GetData(FSCRT_DigestData*& data);

public:
	FSCRT_DigestData* m_pDigestData;
	SHA_CTX m_sSHA_CTX;
};

//Construct and Destruct functions.

CFSCRT_DigestContext::CFSCRT_DigestContext()
{
	m_pDigestData = NULL;
}

CFSCRT_DigestContext::~CFSCRT_DigestContext()
{
	if (NULL!=m_pDigestData)
	{
		delete m_pDigestData;
		m_pDigestData=NULL;
	}
}

//Initialize function.
void CFSCRT_DigestContext::Initialize()
{

}

//Finalize function.
void CFSCRT_DigestContext::Release()
{

}

//Set variable m_pDigestData.
FS_BOOL CFSCRT_DigestContext::SetData(FSCRT_FILE file, const FS_DWORD* byteRangeArray, FS_DWORD sizeofArray)
{
	if (NULL != m_pDigestData)
	{
		delete m_pDigestData;
		m_pDigestData = NULL;
	}

	FSCRT_DigestData* pData = new FSCRT_DigestData;
	pData->m_file = file;
	pData->m_pByteRangeArray = byteRangeArray;
	pData->m_sizeofArray = sizeofArray;

	m_pDigestData = pData;

	return TRUE;
}

//Get variable m_pDigestData.
FS_BOOL CFSCRT_DigestContext::GetData(FSCRT_DigestData*& data)
{
	data = m_pDigestData;
	return TRUE;
}

FS_RESULT getTextFromFile(FSCRT_FILE file, const FS_DWORD *byteRange, unsigned char *plainString)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FSCRT_FILESIZE offset,length, outputSize;
	offset.hiSize = length.hiSize = 0;
	offset.loSize = byteRange[0];
	length.loSize = byteRange[1];
	FSCRT_File_SetRange(file, &offset, &length);

	ret = FSCRT_File_Read(file, plainString, &length, &outputSize);
	if (length.loSize != outputSize.loSize)
		return FSCRT_ERRCODE_ERROR;
	FSCRT_File_ClearRange(file);

	offset.hiSize = length.hiSize = 0;
	offset.loSize = byteRange[2];
	length.loSize = byteRange[3];
	ret = FSCRT_File_SetRange(file, &offset, &length);

	length.loSize = byteRange[3];
	ret = FSCRT_File_Read(file, plainString + outputSize.loSize, &length, &outputSize);
	if (length.loSize != outputSize.loSize)
		return FSCRT_ERRCODE_ERROR;
	FSCRT_File_ClearRange(file);

	return ret;
}


/*
*  Define callback functions to sign document.	
*
*	To sign a document, those callback function will be called in sequence: StartCalcDigest, ContinueCalcDigest, FinishCalcDigest, Sign.
*	To verify the signature, it's order is StartCalcDigest, ContinueCalcDigest, FinishCalcDigest, Verify.
*
*	Developer should be careful to alloc memory in call back function. If you have to use these operation, please remember to release them. 
*	for sha1's handler, CFSCRT_DigestContext finished it's job in function FinishCalcDigest. So it can be released there.
*
*/

FS_RESULT _GlobalStartCalcDigest(FS_LPVOID clientData, FSPDF_SIGNATURE sig, FSCRT_FILE file, const FS_DWORD* byteRangeArray, FS_DWORD sizeofArray, FS_LPVOID* context)
{

	CFSCRT_DigestContext* pContext = new CFSCRT_DigestContext;
	if(!SHA1_Init(&pContext->m_sSHA_CTX))
		return FSCRT_ERRCODE_ERROR;

	pContext->SetData(file, byteRangeArray, sizeofArray);
	*context = pContext;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT _GlobalContinueCalcDigest(FS_LPVOID clientData, FS_LPVOID context, FSCRT_PAUSEHANDLER* pause)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;
	FSCRT_DigestData* pData = NULL;
	pContext->GetData(pData);

	int nFileLength = pData->m_pByteRangeArray[1]+pData->m_pByteRangeArray[3];
	unsigned char* pbFileBuffer = (unsigned char*)malloc(nFileLength);

	getTextFromFile(pData->m_file, pData->m_pByteRangeArray, pbFileBuffer);

	SHA1_Update(&pContext->m_sSHA_CTX, pbFileBuffer, nFileLength);
	free(pbFileBuffer);

	return FSCRT_ERRCODE_FINISHED;
}

FS_RESULT _GlobalFinishCalcDigest(FS_LPVOID clientData, FS_LPVOID context, FSCRT_BSTR* digest)
{
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;
	unsigned char *md = (unsigned char *)OPENSSL_malloc((SHA_DIGEST_LENGTH)*sizeof(unsigned char));

	int len = SHA1_Final(md, &pContext->m_sSHA_CTX);
	if (len != 1)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_Set(digest, (FS_LPCSTR)md, SHA_DIGEST_LENGTH);

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT _GlobalSign_SHA1(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* digest, FSCRT_BSTR* signedData)
{
	TIMESTAMP_PARAM *tsParm = (TIMESTAMP_PARAM *)clientData;
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;
	int nCryptedLength = 0;
	unsigned char *pbCrypted = PKCS7_Sign(tsParm, digest, nCryptedLength);

	if (!pbCrypted)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_SetLength(signedData, nCryptedLength);
	memcpy(signedData->str, pbCrypted, nCryptedLength);
	signedData->len = nCryptedLength;
	free(pbCrypted);
	delete pContext;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT _GlobalVerify_SHA1(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* digest, const FSCRT_BSTR* signedData)
{
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;
	//Usually, the content of a signature field is contain the certification of signer.
	//But we can't judge this certification is trusted. 
	//For this demo, the signer is ourself. So when using api PKCS7_verify to verify, we pass NULL to it's parameter <i>certs</i>.
	//Meanwhile, if application should specify the certificates, we suggest pass flag PKCS7_NOINTERN to api PKCS7_verify.
	FS_BOOL ret = PKCS7_VerifySig(signedData,digest);
	delete pContext;
	return ret ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_ERROR;
}

FS_RESULT _GlobalSign_RFC3161(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* bstrDigest, FSCRT_BSTR* signedData)
{
	TIMESTAMP_PARAM *tsParm = (TIMESTAMP_PARAM *)clientData;
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;

	int nDgLength = 0;

	unsigned char *pbCrypted = PKCS7_Sign(tsParm, bstrDigest, nDgLength, TRUE);

	if (!pbCrypted)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_SetLength(signedData, nDgLength);
	memcpy(signedData->str, pbCrypted, nDgLength);
	signedData->len = nDgLength;
	free(pbCrypted);
	delete pContext;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT _GlobalVerify_RFC3161(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* digest, const FSCRT_BSTR* signedData)
{
	FS_BOOL ret = PKCS7_VerifySig(signedData,digest, TRUE);
	delete context;
	return ret ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_ERROR;
}

FS_RESULT _GlobalSign_Detached(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* digest, FSCRT_BSTR* signedData)
{
	TIMESTAMP_PARAM *tsParm = (TIMESTAMP_PARAM *)clientData;
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;

	int nCryptedLength = 0;

	FSCRT_DigestData* pData = NULL;
	pContext->GetData(pData);

	int nFileLength = pData->m_pByteRangeArray[1]+pData->m_pByteRangeArray[3];
	unsigned char* pbFileBuffer = (unsigned char*)malloc(nFileLength);

	getTextFromFile(pData->m_file, pData->m_pByteRangeArray, pbFileBuffer);

	FSCRT_BSTR plainText = {(FS_LPSTR)pbFileBuffer, (FS_DWORD)nFileLength};

	unsigned char *pbCrypted = PKCS7_Sign(tsParm, &plainText, nCryptedLength);
	free(pbFileBuffer);

	if (!pbCrypted)
		return FSCRT_ERRCODE_ERROR;

	FSCRT_BStr_SetLength(signedData, nCryptedLength);
	memcpy(signedData->str, pbCrypted, nCryptedLength);
	signedData->len = nCryptedLength;
	free(pbCrypted);

	delete pContext;
	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT _GlobalVerify_Detached(FS_LPVOID clientData, FS_LPVOID context, FSPDF_SIGNATURE sig, const FSCRT_BSTR* digest, const FSCRT_BSTR* signedData)
{
	CFSCRT_DigestContext* pContext = (CFSCRT_DigestContext*) context;
	FSCRT_DigestData* pData = NULL;
	pContext->GetData(pData);

	int nFileLength = pData->m_pByteRangeArray[1]+pData->m_pByteRangeArray[3];
	unsigned char* pbFileBuffer = (unsigned char*)malloc(nFileLength);

	getTextFromFile(pData->m_file, pData->m_pByteRangeArray, pbFileBuffer);

	FSCRT_BSTR plainText = {(FS_LPSTR)pbFileBuffer, (FS_DWORD)nFileLength};

	FS_BOOL ret = PKCS7_VerifySig(signedData,&plainText);

	free(pbFileBuffer);
	delete pContext;
	return ret ? FSCRT_ERRCODE_SUCCESS : FSCRT_ERRCODE_ERROR;
}
/*** end call back function***/

/*******************************************************************************/
/*  Create Digital Signature to PDF Document, sign and verify it.              */
/*******************************************************************************/
//Create Digital Signature.
FSPDF_SIGNATURE createSignature(FSCRT_PAGE page, FSCRT_BSTR *subfilter, FS_INT32 firstTime)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;
	FSPDF_SIGNATURE sign = NULL;
	FSCRT_RECTF rect = {120, 110, 220, 10};
	if (firstTime != 0)
	{
		rect.left = 10;
		rect.right = 110;
	}
	//Add an unsigned signature field with blank appearance on the specified position of page.
	ret = FSPDF_Signature_Add(page, &rect, &sign);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		if (ret != FSCRT_ERRCODE_INVALIDLICENSE)
			NO_LICENSE("Signature"); 
		return NULL;
	}
	ret = FSPDF_Signature_SetDefaultContentsLength(sign, 9216);
	//Initialize the value of a signature field.
	ret = FSPDF_Signature_InitValue(sign);
	//Set the name of the preferred signature handler to use when do the signing.
	FSCRT_BSTRC(filter, "Adobe.PPKLite");
	ret = FSPDF_Signature_SetFilter(sign, &filter);
	//Set a name that describes the encoding of the signature value and key information in the signature dictionary.
	ret = FSPDF_Signature_SetSubFilter(sign, subfilter);
	//Set the signer name.
	FSCRT_BSTRC(signer, "james");
	ret = FSPDF_Signature_SetSigner(sign, &signer);
	//Set the text to be showed on signature appearance.
	FSCRT_BSTRC(text, "TestSign");
	ret = FSPDF_Signature_SetText(sign, &text);
	//Set the location of signer.
	FSCRT_BSTRC(location, "FZ");
	ret = FSPDF_Signature_SetLocation(sign, &location);
	//Set the signing reason.
	FSCRT_BSTRC(reason, "Foxit Signature");
	ret = FSPDF_Signature_SetReason(sign, &reason);
	//Set the creation time of signing.
	FSCRT_DATETIMEZONE dt = {2013, 04, 04, 04,17, 18, 18,567,8,0};
	ret = FSPDF_Signature_SetDateTime(sign, &dt);
	//Set the signer distinguished name.
	FSCRT_BSTRC(dn, "CN=CN,MAIL=MAIL@MAIL.COM");
	ret = FSPDF_Signature_SetDistinguishedName(sign, &dn);
	//Set the signature appearance required display appearance.
	ret = FSPDF_Signature_SetAppearanceFlags(sign, 511);

	//Set the icon bitmap in signature visual display.
	FSCRT_IMAGE image = NULL;
	string sFoxitLogoFile = FSDK_GetInputFilesFolder("pdfdigitalsignature");

	void* dir =	FSDK_OpenFolder(sFoxitLogoFile.c_str(),FALSE);
	if(NULL == dir)
	{
		sFoxitLogoFile = FSDK_GetFixFolder();
	}
	else
		FSDK_CloseFolder(dir);

	sFoxitLogoFile += "/FoxitLogo.jpg";

	FSCRT_FILE imagefile = FSDK_OpenFile(sFoxitLogoFile.c_str(), "rb");
	FSCRT_BITMAP bitmap = NULL;
	if (NULL != imagefile)
	{
		FSCRT_Image_LoadFromFile(imagefile, &image);
		FSCRT_Image_LoadFrame(image, 0);
		FSCRT_Image_GetCurrentFrameBitmap(image, &bitmap);
		FSPDF_Signature_SetBitmap(sign, bitmap);	
	}

	ret = FSPDF_Signature_ResetAppearance(sign);

	FSCRT_Bitmap_Release(bitmap);
	FSCRT_Image_Release(image);
	FSCRT_File_Release(imagefile);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		return NULL;
	}

	return sign;
}

FS_RESULT signSignature(FSPDF_SIGNATURE sign, FSCRT_BSTR* fileName)
{
	FS_RESULT ret = FSCRT_ERRCODE_SUCCESS;

	FSCRT_PROGRESS signProgress = NULL;
	FSCRT_FILE savedFile = FSDK_OpenFile(fileName->str, "wb+");
	ret = FSPDF_Signature_StartSign(sign, savedFile, &signProgress);
	if (ret != FSCRT_ERRCODE_FINISHED) {
		if (ret != FSCRT_ERRCODE_SUCCESS) {
			FSCRT_Progress_Release(signProgress);
			FSCRT_File_Release(savedFile);
			return ret;
		}

		ret = FSCRT_ERRCODE_TOBECONTINUED;
		while (ret == FSCRT_ERRCODE_TOBECONTINUED || ret == FSCRT_ERRCODE_ROLLBACK) {
			ret = FSCRT_Progress_Continue(signProgress, NULL);
		}
		FSCRT_Progress_Release(signProgress);
	}
	FSCRT_File_Release(savedFile);

	return (ret == FSCRT_ERRCODE_FINISHED) ? FSCRT_ERRCODE_SUCCESS : ret;
}

FS_RESULT verifySignature(string fileName, FS_INT32 &status)
{
	string log;
	//Open File.
	FSCRT_FILE file = FSDK_OpenFile(fileName.c_str(), "rb");
	if (NULL == file)
		//Failure, output log string and return.
		return FSCRT_ERRCODE_ERROR;

	//Load the PDF document.
	FSCRT_DOCUMENT document = NULL;
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &document, NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all resources allocated.
		FSCRT_File_Release(file);
		return ret;
	}

	FS_INT32 sigCount = 0;
	ret = FSPDF_Doc_CountSignatures(document,&sigCount);

	FSPDF_SIGNATURE sign = NULL;
	if (sigCount == 0)
		return FSCRT_ERRCODE_ERROR;
	ret = FSPDF_Doc_GetSignature(document, sigCount-1, &sign);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all resources allocated.
		FSCRT_File_Release(file);
		return ret;
	}

	//Verify a signature by custom way in progressive manner.
	FSCRT_PROGRESS verifyProgress = NULL;
	ret = FSPDF_Signature_StartVerify(sign, &verifyProgress);
	if (ret == FSCRT_ERRCODE_SUCCESS) {
		//Continue a progressive process.
		ret = FSCRT_ERRCODE_TOBECONTINUED;
		while (ret == FSCRT_ERRCODE_TOBECONTINUED || ret == FSCRT_ERRCODE_ROLLBACK) {
			ret = FSCRT_Progress_Continue(verifyProgress, NULL);
		}
		//Release the current progressive process.
		FSCRT_Progress_Release(verifyProgress);

		ret = FSPDF_Signature_GetState(sign, &status);
		if (ret != FSCRT_ERRCODE_SUCCESS)
			log = "failed to get signature's status\r\n";
	}

	FSCRT_File_Release(file);

	return (ret == FSCRT_ERRCODE_FINISHED) ? FSCRT_ERRCODE_SUCCESS : ret;
	return 0;
}

//for some test files, it's cross reference table is corrupt. Fix it first.
FS_RESULT saveDocToPath(TIMESTAMP_PARAM *param)
{
	FSCRT_FILE file;

	FS_RESULT ret = FSCRT_File_CreateFromFileName(&param->inputFile, FSCRT_FILEMODE_READONLY, &file);
	if (!file) return FSCRT_ERRCODE_ERROR;

	FSCRT_DOCUMENT pdfDoc = NULL;
	ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (ret != FSCRT_ERRCODE_SUCCESS)return ret;

	string pdffile = FSDK_GetFileName(param->inputFile.str);
	string outputFilesPath = FSDK_GetOutputFilesFolder("pdfdigitalsignature");
	string newFilePath = outputFilesPath+"/newInput_" + pdffile;

	FSCRT_FILE saveFile;
	FSCRT_BSTR path;
	FSCRT_BStr_Init(&path);
	FSCRT_BStr_Set(&path,newFilePath.c_str(),newFilePath.length());
	ret = FSCRT_File_CreateFromFileName(&path, FSCRT_FILEMODE_TRUNCATE, &saveFile);
	if (ret != FSCRT_ERRCODE_SUCCESS)return ret;

	FSCRT_PROGRESS progress;
	ret = FSPDF_Doc_StartSaveToFile(pdfDoc, saveFile, FSPDF_SAVEFLAG_NOORIGINAL, &progress);
	if (ret != FSCRT_ERRCODE_SUCCESS)return ret;

	ret = FSCRT_ERRCODE_TOBECONTINUED;
	while(ret == FSCRT_ERRCODE_TOBECONTINUED)
		ret = FSCRT_Progress_Continue(progress, NULL);

	if (ret != FSCRT_ERRCODE_FINISHED) return FSCRT_ERRCODE_ERROR;

	FSCRT_Progress_Release(progress);
	FSPDF_Doc_Close(pdfDoc);

	FSCRT_File_Release(file);
	FSCRT_File_Release(saveFile);

	FSCRT_BStr_Set(&param->inputFile, newFilePath.c_str(), newFilePath.length());
	FSCRT_BStr_Clear(&path);

	return FSCRT_ERRCODE_SUCCESS;
}

FS_RESULT signAndVerifyProc(TIMESTAMP_PARAM *param, FS_BOOL isFirstTime)
{
	FS_RESULT ret  = FSCRT_ERRCODE_SUCCESS;
	FSDK_OutputLog("\tInput File: %s\r\n", param->inputFile.str);
	FSDK_OutputLog("\tOutput to: %s\r\n",param->outputFile.str);

	//for some test files, their cross reference tables are corrupted. Fix it first.
	if (isFirstTime){
		ret = saveDocToPath(param);
		if (ret != FSCRT_ERRCODE_SUCCESS)
		{
			FSDK_OutputLog("can't load file: %s\r\n", param->inputFile.str);
			return ret;
		}
	}

	string strLog("\r\n\t");
	strLog += param->inputFile.str;

	strLog += "\r\n\topen file: ";
	FSCRT_FILE file = FSDK_OpenFile(param->inputFile.str, "rb");
	if (!file)
	{
		strLog += "failure\r\n";
		FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());
		return FSCRT_ERRCODE_ERROR;
	}
	strLog += "success\r\n";

	strLog += "\tload PDF document: ";
	FSCRT_DOCUMENT pdfDoc = NULL;
	ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		strLog += "success\r\n";

		FS_INT32 pageIndex = 0;
		char* buff = new char[128];
		memset(buff, 0, sizeof(char)*128);
		sprintf(buff, "%d", pageIndex);
		string tmp = buff;
		delete []buff;
		buff = NULL;
		strLog += "\tload page " + tmp + " ";
		FSCRT_PAGE pdfPage = NULL;
		ret = FSPDF_Doc_GetPage(pdfDoc, pageIndex, &pdfPage);
		if (ret == FSCRT_ERRCODE_SUCCESS)
		{
			strLog += "success\r\n";

			strLog += "\tCreate Signature: ";
			FSPDF_SIGNATURE sign = createSignature(pdfPage, &param->subfilter,isFirstTime);
			if (NULL != sign)
			{
				strLog += "success\r\n";
				strLog += "\tSign Signature: ";

				ret = signSignature(sign, &param->outputFile);
				if (ret == FSCRT_ERRCODE_SUCCESS)
				{
					strLog += "success\r\n";
					strLog += "\tVerifying Signature:";

					FS_INT32 signStatus = -1;
					ret = verifySignature(param->outputFile.str, signStatus);

					if (ret == FSCRT_ERRCODE_SUCCESS)
						strLog += "success\r\n";
					else 
						strLog += "failure\r\n";

					if (signStatus != FSPDF_SIGNATURE_STATE_VALID)
						strLog += "\r\n\tsignature's status is INVALID!\r\n";
					else 
						strLog += "\r\n\tsignature's status is VALID!\r\n";
				}
				else	
					strLog += "failure\r\n";
			}// end if (NULL != sign)
			else
				//Fails when create signature.
				strLog += "failure\r\n";

			//Release all page content objects.
			FSPDF_Page_Clear(pdfPage);
			pdfPage = NULL;
		}
		else
			//Fails when load PDF page.
			strLog += "failure\r\n";
		//Close the PDF document.
		FSPDF_Doc_Close(pdfDoc);
		pdfDoc = NULL;
	}//end if (ret == FSCRT_ERRCODE_SUCCESS)
	else
		//Fails when load PDF file.
		strLog += "failure\r\n";

	FSCRT_File_Release(file);

	FSDK_OutputStringLog(strLog.c_str(), (int)strLog.length());

	if (isFirstTime){
		//remove the fixed pdf file. If user want to check what's modified, do not remove it.
		remove(param->inputFile.str);
	}
	return FSCRT_ERRCODE_SUCCESS;
}

void initLogEnv(TIMESTAMP_PARAM* tsParam)
{
	string logPath ;

	logPath = tsParam->outputFile.str;
	logPath = logPath.substr(0, logPath.rfind("/")+1);
	FSDK_CreatFolder(logPath.c_str());

	logPath += "log.txt";

	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfdigitalsignature\r\n");
}


void outputCommandUsage()
{
	printf("\r\n");
	printf("usage: pdfdigitalsignature [/i inputfile] [/sf subfilter] [/o destfolder] [/t sig/doc] [/u url] [/up user:pswd] [/p port]\r\n");
	printf("The Default certificate is foxit_all.pfx. And this demo will sign a doc twice.\r\n\r\n");
	printf("/i inputfile\t: (optional)the document which the time stamp will insert to.\r\n");
	printf("/sf subfilter\t: (optional)specify the subfilter of signature, one of\r\n\t\t  \"adbe.pkcs7.sha1\", \"adbe.pkcs7.detached\", and \"ETSI.RFC3161\".\r\n");
	printf("/o destfolder\t: (optional)path to store the output file.\r\n\t\t  can not be same with prefix of input file\r\n");
	printf("/t sig/doc\t: (optional)\"sig\" means to add a time stamp to a signature.\r\n\t\t  \"doc\" means add a time stamp to document.\r\n");
	printf("/u url\t\t: (required if /t present)the url of TSA server\r\n");
	printf("/up user:pswd\t: (optional)user name and password to TSA server\r\n");
	printf("/p port\t\t: (optional)TSA server port\r\n\r\n");
}

/*******************************************************************************/
/*  Defines the entry point for the console application.					   */
/*******************************************************************************/

int main(int argc, char* argv[])
{
	FSCRT_FILE gsdkLogFile = NULL;
	FS_RESULT nRet = FSCRT_ERRCODE_SUCCESS;
	FSDK_SetExecuteFilePath(argc, argv);

	//Initialize SDK library, or no method of SDK can be used.
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		FSDK_OutputLog("failure\r\n");
		FSDK_CloseLog();
		return -1;
	}

	TIMESTAMP_PARAM param;
	memset(&param, 0, sizeof(TIMESTAMP_PARAM));
	if (!parseParam(argc, argv, &param))
	{
		outputCommandUsage();
		clearParam(&param);
		//Finalize SDK library.
		FSDK_FinalizeLibrary();
		//Close log file.
		FSDK_CloseLog();

		ERR_free_strings();
		return -1;
	}
	if (!validateParam(&param))
	{
		outputCommandUsage();
		clearParam(&param);
		//Finalize SDK library.
		FSDK_FinalizeLibrary();
		//Close log file.
		FSDK_CloseLog();

		ERR_free_strings();
		return -1;
	}

	initLogEnv(&param);
	initialize_openssl();

	nRet = FSCRT_PDFModule_Initialize();
	if (nRet == FSCRT_ERRCODE_SUCCESS)
	{
		string outputFilesPath = FSDK_GetOutputFilesFolder("pdfdigitalsignature");

		FS_BOOL bRet = FSDK_SetLibraryLogFile(outputFilesPath + "/gsdkLog.txt", gsdkLogFile);
		if (!bRet)
		{
			clearParam(&param);
			//Finalize SDK library.
			FSDK_FinalizeLibrary();
			//Close log file.
			FSDK_CloseLog();

			ERR_free_strings();
			return -1;
		}

		// this handler specify the digest and sign callback function. Usually, call back function is different between each other,
		// but for this demo, it can be same.
		FSPDF_SIGNATUREHANDLER sha1Handler;
		sha1Handler.clientData = (FS_LPVOID)&param;
		sha1Handler.StartCalcDigest = _GlobalStartCalcDigest;
		sha1Handler.ContinueCalcDigest = _GlobalContinueCalcDigest;
		sha1Handler.FinishCalcDigest = _GlobalFinishCalcDigest;
		sha1Handler.Sign = _GlobalSign_SHA1;
		sha1Handler.Verify = _GlobalVerify_SHA1;
		FSCRT_BSTRC(subfilter_sha1, "adbe.pkcs7.sha1");

		FSPDF_SIGNATUREHANDLER detachedHandler;
		detachedHandler.clientData = (FS_LPVOID)&param;
		detachedHandler.StartCalcDigest = _GlobalStartCalcDigest;
		detachedHandler.ContinueCalcDigest = _GlobalContinueCalcDigest;
		detachedHandler.FinishCalcDigest = _GlobalFinishCalcDigest;
		detachedHandler.Sign = _GlobalSign_Detached;
		detachedHandler.Verify = _GlobalVerify_Detached;
		FSCRT_BSTRC(subfilter_detached, "adbe.pkcs7.detached");

		FSPDF_SIGNATUREHANDLER RFC3161Handler;
		RFC3161Handler.clientData = (FS_LPVOID)&param;
		RFC3161Handler.StartCalcDigest = _GlobalStartCalcDigest;
		RFC3161Handler.ContinueCalcDigest = _GlobalContinueCalcDigest;
		RFC3161Handler.FinishCalcDigest = _GlobalFinishCalcDigest;
		RFC3161Handler.Sign = _GlobalSign_RFC3161;
		RFC3161Handler.Verify = _GlobalVerify_RFC3161;
		FSCRT_BSTRC(subfilter_rfc3161, "ETSI.RFC3161");

		FSCRT_BSTRC(filter, "Adobe.PPKLite");
		nRet = FSPDF_Signature_RegisterHandler(&filter, &subfilter_sha1, &sha1Handler);
		nRet = FSPDF_Signature_RegisterHandler(&filter, &subfilter_detached, &detachedHandler);
		nRet = FSPDF_Signature_RegisterHandler(&filter, &subfilter_rfc3161, &RFC3161Handler);

		//Create an unsigned digital signature on the specified position of PDF document, sign and verify it.
		FSDK_OutputLog("\r\nSign and verify the document:\r\n");
		FS_RESULT nRet= signAndVerifyProc(&param, TRUE);

		if (nRet == FSCRT_ERRCODE_SUCCESS)
			FSDK_OutputLog("\r\nsignature demo exit normal.\r\n\r\n");
		else
			FSDK_OutputLog("\r\nsignature demo exit with error.\r\n\r\n");

		//use other filter to sign the output file.
		switchSubfilter(&param);

		FSDK_OutputLog("\r\nSign and verify the document:\r\n");
		nRet= signAndVerifyProc(&param, FALSE);

		if (nRet == FSCRT_ERRCODE_SUCCESS)
			FSDK_OutputLog("\r\nsignature demo exit normal.\r\n\r\n");
		else
			FSDK_OutputLog("\r\nsignature demo exit with error.\r\n\r\n");

	}
	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	clearParam(&param);
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();

	ERR_free_strings();

	return 0;
}
