/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This file is an example to get page label information of a PDF page in a PDF document.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfpagelabel
#endif

///This function is to get page label information
FS_RESULT FSDK_GetPageLabelInfo(string strFile, string inputFilesPath)
{
	//Begin, output log 
	FSDK_OutputLog("\r\n====Get page label====\r\n");

	//Get inputting file name and output log 
	FSDK_OutputLog("**");
	FSDK_OutputStringLog(strFile.c_str(), (int)strFile.length());
	FSDK_OutputLog("**\r\n");
	
	unsigned int nStartTime = FSDK_GetClock();//Get current time
	//Open file
	FSCRT_FILE file = FSDK_OpenFile((inputFilesPath + "/" + strFile).c_str(), "rb");
	if (!file)
	{
		//Failure, output log and return
		FSDK_OutputLog("\tOpen file failure!\r\n");
		return FSCRT_ERRCODE_ERROR;
	}
	//Open successfully
	FSDK_OutputLog("\tOpen file success.\r\n");

	//Load PDF file
	FSCRT_DOCUMENT pdfDoc = NULL;
	FS_RESULT ret = FSPDF_Doc_StartLoad(file, NULL, &pdfDoc, NULL);
	if (ret == FSCRT_ERRCODE_SUCCESS)
	{
		//Load successfully
		FSDK_OutputLog("\tLoad PDF document success.\r\n");
		//Get count of PDF pages
		FS_INT32 pageCount = 0;
		ret = FSPDF_Doc_CountPages(pdfDoc, &pageCount);
		if (ret == FSCRT_ERRCODE_SUCCESS)
		{
			FS_INT32 pageIndex = 0;
			do
			{
				//Define a ::FSCRT_BSTR object to receive page label information
				FSCRT_BSTR pageLabel;
				FSCRT_BStr_Init(&pageLabel);//Initialize file name
				//Convert a page index to page label
				ret = FSPDF_Doc_PageIndexToPageLabel(pdfDoc, pageIndex, &pageLabel);
				if (ret == FSCRT_ERRCODE_SUCCESS)
				{
					//Output log of page index and page label
					string cPageLabel(pageLabel.str,pageLabel.len);
					FSDK_OutputLog("\t Index: %d,\tPageLabel: ", pageIndex);
					FSDK_OutputStringLog(cPageLabel.c_str(), (int)cPageLabel.length());
					FSDK_OutputLog("\r\n");
					//Clear page label
					FSCRT_BStr_Clear(&pageLabel);
				}
				//Move to next page
				pageIndex ++;
			} while (pageIndex < pageCount);
		}
		//Close PDF document
		FSPDF_Doc_Close(pdfDoc);

		//Get current time and output elapse time
		unsigned int nEndTime = FSDK_GetClock();
		FSDK_OutputLog("\tPage count: %d\r\n\telapsed time: %dms\r\n", pageCount, nEndTime - nStartTime);
	}
	else	//Fails when load PDF file
	{
		FSDK_OutputLog("\tLoad PDF document failure!\r\n");
		FSDK_CHECK_LICENSE(ret);
	}
	
	FSCRT_File_Release(file);//Release file object
	//End, output log 
	FSDK_OutputLog("====End get page label====\r\n");

	return ret;
}

void FSDK_GetPageLabelFolder(void* folder, string inputFilesPath)
{
	FSDK_OutputLog("success.\r\n");

	do
	{
		//Get a file from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);

		if(!isDir && FSDK_CompareExtName(strFile.c_str(), ".pdf")) 
			FSDK_GetPageLabelInfo(strFile, inputFilesPath);
		
		//If it's an invalid file name
		if (strFile.size() < 1) break;
	} while (1);
}

int main(int argc, char* argv[])
{
	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	string inputSrcFile = "";
	string outputFolder = "";
	//Judge if to command line arguments. 
	//usage: pdfpagelabel <srcfile> /o [destfolder]
	if (argc > 1)
	{
		inputSrcFile = argv[1];
		bool bArgumentValid = true;
		if (argc >= 3)
		{
			if (0 != strcmp(argv[2], "/o"))
				bArgumentValid = false;
			else if (argc > 3)
				outputFolder = argv[3];
		}
		if (false == bArgumentValid || !FSDK_FileExist(inputSrcFile.c_str()) || (outputFolder != "" && !FSDK_FolderExist(outputFolder.c_str())))
		{
			printf("\r\n");
			printf("usage: pdfpagelabel [<srcfile> /o [destfolder]] \r\n\r\n");
			printf("<srcfile>\t: Path of an image file as input file.\r\n");
			printf("/o [destfolder]\t: Path of a folder as output folder. If not set, use default\r\n\t\t  output folder.\r\n\r\n");
			return -1;
		}
	}
	// Locate the path of log file
	string logPath ;
	if (outputFolder != "")
	{
		outputFolder += "/pdfpagelabel";
		FSDK_CreatFolder(outputFolder.c_str());
		logPath = outputFolder;
		logPath += "/log.txt";
	}
	else
	{
		outputFolder = FSDK_GetOutputFilesFolder("pdfpagelabel");
		logPath = FSDK_GetLogFile("pdfpagelabel");
	}
	
	//Open and write log
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfpagelabel\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Fail to initialize, output status and return
		FSDK_OutputLog("failure!\r\n");
		FSDK_CloseLog();
		return -1;
	}
	FSDK_OutputLog("success.\r\n");

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(outputFolder + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	//Get folder of input files for this example
	string inputFolder;
	string pdfName;
	if (inputSrcFile != "")
	{
		pdfName = FSDK_GetFileName(inputSrcFile.c_str());
		inputFolder = FSDK_GetCustomInputPath(inputSrcFile.c_str());
		FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
		FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
		FSDK_GetPageLabelInfo(pdfName, inputFolder);
	}
	else
	{
		inputFolder = FSDK_GetInputFilesFolder("pdfpagelabel");
		//Open folder to enumerate files
		void* folder = FSDK_OpenFolder(inputFolder.c_str());
		if (folder)
		{
			FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
			FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfpagelabel: ");
			FSDK_GetPageLabelFolder(folder, inputFolder);
			//Close access to specific folder.
			FSDK_CloseFolder(folder);
		}
		else
		{
			inputFolder = FSDK_GetFixFolder();
			FSDK_OutputLog("Input from folder %s\r\n", inputFolder.c_str());
			FSDK_OutputLog("Output to folder %s \r\n\r\n", outputFolder.c_str());
			pdfName  = "PageLabels.pdf";
			FSDK_GetPageLabelInfo(pdfName, inputFolder);
		}
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();

	//getchar();
	return 0;
}

