/**
 * Copyright (C) 2001-2017, Foxit Software Inc..
 * All Rights Reserved.
 *
 * http://www.foxitsoftware.com
 *
 * The following code is copyrighted and contains proprietary information and trade secrets of Foxit Software Inc..
 * You cannot distribute any part of Foxit PDF SDK to any third party or general public, 
 * unless there is a separate license agreement with Foxit Software Inc. which explicitly grants you such rights.
 *
 * This demo illustrates how to use Foxit PDF SDK to create a PDF, insert the four supported page objects into the created PDF file, 
 * and save the file with page objects already inserted.
 */

//Include common header file.
#include "../comm_src/fgsdk_common.h"

#if defined(FSDK_IOS_DEMO)
#define main pdfpageobjects
#endif

#define  PageWidth             612                      //The new page width
#define  PageHeight            792                      //The new page height

class CPageObjects
{
public:
	CPageObjects();
	~CPageObjects();
	void  PageObjectsFolder(void* folder);
	void  PageObjectsInfo(string strFile);

public:
	string m_strInputPath;
	string m_strOutputPath;
};

CPageObjects::CPageObjects()
	:m_strInputPath("")
	,m_strOutputPath("")
{

}

CPageObjects::~CPageObjects()
{

}

//Create path data used by Path Object and Form XObject Object.
FS_RESULT CreatePathData(FSCRT_PATHDATA* pathData)
{
	//Create path data
	FS_RESULT ret = FSCRT_PathData_Create(pathData);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		FSDK_OutputLog("Failure: In function FSCRT_PathData_Create with error code %d.\r\n", ret);
	}
	else
	{
		FSDK_OutputLog("Success: Create path data used by Path Object and Form XObject Object.\r\n");
	}
	return ret;
}

//Insert a Text Object
void InsertTextObject(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs)
{ 
	char*  TextObjFontName = (char*)"Helvetica";              //Font name used by Text Object
	char*  TextObjUnicodeStr = (char*)"Insert a Text Object";   //Text Object unicode string

	//Get the font name
	FSCRT_BSTR NewfontName = {TextObjFontName, (FS_DWORD)strlen(TextObjFontName)};

	//Create a font object
	FSCRT_FONT creatFont = NULL;
	FS_RESULT ret = FSCRT_Font_Create(&NewfontName, FSCRT_FONTSTYLE_SERIF, 5, 0, &creatFont);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		//Cannot create a font
		FSDK_OutputLog("Failure: In function FSCRT_Font_Create with error code %d.\r\n", ret);
		return;
	}

	//Define text states
	FSPDF_TEXTSTATE textStateTemp;
	textStateTemp.charSpace = 0;
	textStateTemp.font = creatFont;//Font object
	textStateTemp.fontSize = 8.50;
	textStateTemp.textMatrix[0] = 4;
	textStateTemp.textMatrix[1] = 10;
	textStateTemp.textMatrix[2] = 1;
	textStateTemp.textMatrix[3] = 10;
	textStateTemp.textMode = 0;
	textStateTemp.wordSpace = 0;
	textStateTemp.x = 0;
	textStateTemp.y = 0;
	FS_BOOL italic = TRUE;//Text is italic
	FS_INT32 weight = 400; //Font weight

	//Create a text object
	CAutoPageObject atuoTextObj;
	ret = atuoTextObj.CreateTextObject(page);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release the font object
		FSCRT_Font_Release(creatFont);
		return;
	}

	//Set text states to the text object.
	ret = FSPDF_TextObject_SetTextState(page, atuoTextObj.GetObject() ,&textStateTemp, italic, weight);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release the font object
		FSCRT_Font_Release(creatFont);
		FSDK_OutputLog("Failure: In function FSPDF_TextObject_SetTextState with error code %d.\r\n", ret);
		return;
	}

	//Set unicode string to the text object.
	FSCRT_BSTR unicodeStr = {TextObjUnicodeStr, (FS_DWORD)strlen(TextObjUnicodeStr)};//Get the text object unicode string.	
	ret = FSPDF_TextObject_SetUnicodeString(page, atuoTextObj.GetObject(), &unicodeStr);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release the font object
		FSCRT_Font_Release(creatFont);
		FSDK_OutputLog("Failure: In function FSPDF_TextObject_SetUnicodeString with error code %d.\r\n", ret);
		return;
	}
	FSCRT_MATRIX matrixCurrent ;
	FSPDF_PageObject_GetMatrix(page,atuoTextObj.GetObject(), &matrixCurrent);
	//Set matrix to the text object.
	FSCRT_MATRIX textmatrix = {1, 0, 0, 1, 100, 100};
	FSCRT_Matrix_Translate(&matrixCurrent,textmatrix.e,textmatrix.f);	
	ret = FSPDF_PageObject_SetMatrix(page, atuoTextObj.GetObject(), &matrixCurrent);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release the font object
		FSCRT_Font_Release(creatFont);
		FSDK_OutputLog("Failure: In function FSPDF_PageObject_SetMatrix with error code %d.\r\n", ret);
		return;
	}
	//Insert the text object and generate PDF page contents.
	atuoTextObj.InsertGenerateContents(page, pageObjs, FSPDF_PAGEOBJECT_TEXT, 0);
	//Release the font object
	FSCRT_Font_Release(creatFont);
	FSDK_OutputLog("Success: Insert a TextObject to page.\r\n");
}

//Insert a Image Object
void InsertImageObject(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs, string inputFiles)
{
	//Open image file
	CAutoImage autoImage(inputFiles.c_str(), "rb");
	if (!autoImage.GetFile()) return;

	//Load image from image file
	FS_RESULT ret = autoImage.LoadImage();
	if (ret != FSCRT_ERRCODE_SUCCESS) return;

	//Count frames of image
	int count = 0;
	ret = FSCRT_Image_CountFrames(autoImage.GetImage(), &count);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSDK_OutputLog("failure. In function FSCRT_Image_CountFrames with error code %d.\r\n", ret);
		return;
	}

	if (count > 0) {
		//Load the last image frames.
		int frameIndex = count - 1;//Frame index
		ret = FSCRT_Image_LoadFrame(autoImage.GetImage(), frameIndex);
		if (ret != FSCRT_ERRCODE_SUCCESS) 
		{
			FSDK_OutputLog("failure. In function FSCRT_Image_LoadFrame with error code %d.\r\n", ret);
			return;
		}

		CAutoPageObject autoImageObj;
		ret = autoImageObj.CreateImageObject(page);
		if (ret != FSCRT_ERRCODE_SUCCESS) 
		{
			return;
		}
		//Set an image to the image object.
		ret = FSPDF_ImageObject_SetImage(page, autoImageObj.GetObject(), autoImage.GetImage());
		if (ret != FSCRT_ERRCODE_SUCCESS) 
		{
			FSDK_OutputLog("failure. In function FSPDF_ImageObject_SetImage with error code %d.\r\n", ret);
			return;
		}
		//Set matrix to the image object.
		FSCRT_MATRIX imageMatrix = {150, 0, 0, 80 , 100, 320};
		ret = FSPDF_PageObject_SetMatrix(page, autoImageObj.GetObject(), &imageMatrix);
		if (ret != FSCRT_ERRCODE_SUCCESS) 
		{
			FSDK_OutputLog("failure. In function FSPDF_PageObject_SetMatrix with error code %d.\r\n", ret);
			return;
		}
		//Insert the image object and generate PDF page contents.
		autoImageObj.InsertGenerateContents(page, pageObjs, FSPDF_PAGEOBJECT_ALL, 1);
	}
	FSDK_OutputLog("Success: Insert a Image Object to page.\r\n");
}

//Insert a Path Object
void InsertPathObject(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs)
{
	//Create path data
	FSCRT_PATHDATA pathData = NULL;
	FS_RESULT ret = CreatePathData(&pathData);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;//Create path data failure and return

	FSCRT_PathData_MoveTo(pathData, 100.0f, 500.0f);//Add a point(100.0f, 500.0f) to start a figure.
	FSCRT_PathData_LineTo(pathData, 300.0f, 500.0f);//Add a point(300.0f, 500.0f) to the figure, the previous point(100.0f, 500.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathData, 300.0f, 450.0f);//Add a point(300.0f, 450.0f) to the figure, the previous point(300.0f, 500.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathData, 100.0f, 450.0f);//Add a point(100.0f, 450.0f) to the figure, the previous point(300.0f, 450.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathData, 100.0f, 500.0f);//Add a point(100.0f, 500.0f) to the figure, the previous point(100.0f, 450.0f) and the point are the endpoints of a line.

	//Create a Path Object
	CAutoPageObject autoPathObj;
	ret = autoPathObj.CreatePathObject(page);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathData);
		return;
	}

	//Set path data to the path object
	ret = FSPDF_PathObject_SetPathData(page, autoPathObj.GetObject(), pathData);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathData);
		FSDK_OutputLog("failure. In function FSPDF_PathObject_SetPathData with error code %d.\r\n", ret);
		return;
	}
	//Insert the path object and generate PDF page contents.
	autoPathObj.InsertGenerateContents(page, pageObjs, FSPDF_PAGEOBJECT_ALL, 2);

	//Release all related resources of the path data.
	FSCRT_PathData_Release(pathData);
	FSDK_OutputLog("Success: Insert a Path Object.\r\n");
	return;
}

//Insert a Form XObject Object
void InsertFormXObject(FSCRT_PAGE page, FSPDF_PAGEOBJECTS pageObjs)
{
	//Create path data
	FSCRT_PATHDATA pathDataClone = NULL;
	FS_RESULT ret = CreatePathData(&pathDataClone);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;//Create path data failure and return

	FSCRT_PathData_MoveTo(pathDataClone, 100.0f, 100.0f);//Add a point(100.0f, 100.0f) to start a figure.
	FSCRT_PathData_LineTo(pathDataClone, 300.0f, 100.0f);//Add a point(300.0f, 100.0f) to the figure, the previous point(100.0f, 500.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathDataClone, 300.0f, 150.0f);//Add a point(300.0f, 150.0f) to the figure, the previous point(300.0f, 100.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathDataClone, 100.0f, 150.0f);//Add a point(100.0f, 150.0f) to the figure, the previous point(300.0f, 150.0f) and the point are the endpoints of a line.
	FSCRT_PathData_LineTo(pathDataClone, 100.0f, 100.0f);//Add a point(100.0f, 100.0f) to the figure, the previous point(100.0f, 150.0f) and the point are the endpoints of a line.

	//Create a path object
	FSPDF_PAGEOBJECT clonePath = NULL;
	ret = FSPDF_PathObject_Create(page, &clonePath);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathDataClone);
		FSPDF_PageObject_Release(page,clonePath);
		FSDK_OutputLog("failure. In function FSPDF_PathObject_Create with error code %d.\r\n", ret);
		return;
	}

	//Set path data to the path object
	ret = FSPDF_PathObject_SetPathData(page, clonePath, pathDataClone);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathDataClone);
		FSPDF_PageObject_Release(page,clonePath);
		FSDK_OutputLog("failure. In function FSPDF_PathObject_SetPathData with error code %d.\r\n", ret);
		return;
	}

	//Create a Form XObject object
	CAutoPageObject autoFormXObj;
	ret = autoFormXObj.CreateFormXObject(page);

	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathDataClone);
		FSPDF_PageObject_Release(page,clonePath);
		return;
	}

	//Set matrix to the form Xobject.
	FSCRT_MATRIX formXObjMatrix = {1, 0, 0, 1.5 , 0, 0};
	ret = FSPDF_PageObject_SetMatrix(page, autoFormXObj.GetObject(), &formXObjMatrix);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathDataClone);
		FSPDF_PageObject_Release(page,clonePath);
		FSDK_OutputLog("failure. In function FSPDF_PageObject_SetMatrix with error code %d.\r\n", ret);
		return;
	}
	//Get page objects(formXObjs) from the form Xobject(formXObjs).
	FSPDF_PAGEOBJECTS formXObjs = NULL;
	ret = FSPDF_FormXObject_GetObjects(page, autoFormXObj.GetObject(), &formXObjs);
	if (ret != FSCRT_ERRCODE_SUCCESS)
	{
		//Release all related resources of the path data.
		FSCRT_PathData_Release(pathDataClone);
		FSPDF_PageObject_Release(page,clonePath);
		FSDK_OutputLog("failure. In function FSPDF_FormXObject_GetObjects with error code %d.\r\n", ret);
		return;
	}

	//Insert the form Xobject and generate PDF page contents.
	autoFormXObj.InsertGenerateContents(page, pageObjs, FSPDF_PAGEOBJECT_ALL, 3);

	//Release all related resources of the path data.
	FSCRT_PathData_Release(pathDataClone);
	FSPDF_PageObject_Release(page,clonePath);
	return;
}

//Create document and page, insert page objects.
void CPageObjects::PageObjectsInfo(string strFile)
{
	if (!FSDK_IsSupportedImageFile(strFile.c_str())) return;
	//Create a new document.
	CAutoDocument autoDoc;
	FS_RESULT ret = autoDoc.CreatePDFDocument();
	if (ret != FSCRT_ERRCODE_SUCCESS) return;

	//Create a new page
	CAutoPDFPage autoPage;
	ret = autoPage.CreatePDFPage(autoDoc.GetDocument(), 0);
	if (ret != FSCRT_ERRCODE_SUCCESS) return;

	ret = FSPDF_Page_SetSize(autoPage.GetPDFPage(), PageWidth, PageHeight);
	if (ret != FSCRT_ERRCODE_SUCCESS) 
	{
		FSDK_OutputLog("Failure: In function FSPDF_Page_SetSize with error code %d.\r\n", ret);
		return;
	}

	ret = autoPage.ParsePage();
	if (ret != FSCRT_ERRCODE_SUCCESS) return;

	//Get page objects in a page
	FSPDF_PAGEOBJECTS pageObjs;
	FSPDF_Page_GetPageObjects(autoPage.GetPDFPage(), &pageObjs);

	//Insert a text object to the page
	InsertTextObject(autoPage.GetPDFPage(), pageObjs);

	//Insert a image object to the page
	InsertImageObject(autoPage.GetPDFPage(), pageObjs, m_strInputPath + strFile);

	//Insert a path object to the page
	InsertPathObject(autoPage.GetPDFPage(), pageObjs);

	//Insert a form Xobject to the page
	InsertFormXObject(autoPage.GetPDFPage(), pageObjs);

	//Get the output file name to save the document
	string bmpfile(strFile.c_str(), strFile.length() - 4);
	string filename =  "pdfPageObjects_" + bmpfile + "_Test" + ".pdf";

	//Save the document
	FSDK_SavePDFFile(autoDoc.GetDocument(), (m_strOutputPath + "/" + filename).c_str());
	FSDK_OutputLog("Success: Insert page objects to page.\r\n");
}

void CPageObjects::PageObjectsFolder(void* folder)
{
	do
	{
		//Get a file object from folder
		FS_BOOL isDir = FALSE;
		string strFile = FSDK_GetNextFile(folder, &isDir);
		if (strFile.size() > 0 && !isDir)
		{
			PageObjectsInfo(strFile);
		}

		//If it's an invalid file name
		if (strFile.size() < 1) break;
	} while (1);
}

//Entrance of this example
int main(int argc, char* argv[])
{

	//Set executing file path
	FSDK_SetExecuteFilePath(argc, argv);

	//Open log file and output some basic info
	string logPath = FSDK_GetLogFile("pdfpageobjects");
	FSDK_OpenLog(logPath.c_str());
	FSDK_OutputLog("Foxit PDF SDK example: pdfpageobjects\r\n");
	FSDK_OutputLog("\tInput from: input_files folder\r\n");
	FSDK_OutputLog("\tOutput to: output_files/pdfpageobjects\r\n\r\n");

	//Initialize Foxit SDK library
	bool bRet = FSDK_InitializeLibray(true);
	if (!bRet)
	{
		//Failed to initialize, output status and return
		FSDK_CloseLog();//Close log
		return -1;
	}

	//Initialize PDF module
	if(!FSDK_PDFModule_Initialize()){
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	CPageObjects pageObjects;
	//Get output files folder path.
	pageObjects.m_strOutputPath = FSDK_GetOutputFilesFolder("pdfpageobjects");
	FSCRT_FILE gsdkLogFile = NULL;
	bRet = FSDK_SetLibraryLogFile(pageObjects.m_strOutputPath + "/gsdkLog.txt", gsdkLogFile);
	if (!bRet)
	{
		FSDK_PDFModule_Finalize();
		FSDK_FinalizeLibrary();
		FSDK_CloseLog();
		return -1;
	}

	pageObjects.m_strInputPath = FSDK_GetInputFilesFolder("pdfpageobjects");
	//Open folder to enumerate files
	void* folder = FSDK_OpenFolder(pageObjects.m_strInputPath.c_str());

	if (folder)
	{
		//Get folder of inputting files for this example
		FSDK_OutputLog("\r\nEnumerate files in folder input_files/pdfpageobjects: ");
		pageObjects.PageObjectsFolder(folder);
		//Close access to specific folder.
		FSDK_CloseFolder(folder);
	}
	else
	{
		pageObjects.m_strInputPath = FSDK_GetFixFolder();
		pageObjects.m_strInputPath = pageObjects.m_strInputPath +"/";
		string bitmapName = "Foxit.bmp";
		pageObjects.PageObjectsInfo(bitmapName);
	}

	if (gsdkLogFile)
		FSCRT_File_Release(gsdkLogFile);

	//Finalize PDF module.
	FSDK_PDFModule_Finalize();
	//Finalize SDK library.
	FSDK_FinalizeLibrary();
	//Close log file.
	FSDK_CloseLog();
	return 0;
}
